/*	The purpose of this program is, to save
 * a list of signed integers in binary format,
 * the least significant 14 bits of which are taken
 * as fractional, but that complete a sine wave over
 * 256 elements, with max amplitude of +/- 2^14 .
 *
 * 	This table is meant to be included as an intializer list
 * so that another program can dereference it,
 * and arrive at approximations of the sine function,
 * of a 16-bit integer parameter, the modulus of which
 * completes one full cycle.
 *
 * 	The choice of a 14-bit fraction will make it possible
 * for every element of the array to be stored as a
 * 16-biit integer.
 *
 */

#include <stdio.h>
#include <math.h>
#include <stdint.h>

int main(int argc, char* argv[])
{
    double temp = 0.0;
    int a = 0;

    int16_t s = 0x8000;
    uint16_t u = 0x8000;

    s = s >> 15;
    u = u >> 15;

    printf("The signed integer was right-shifted to become %d\n", s);
    printf("The unsigned integer was right-shifted to become %d\n\n", u);

    if (s == -1 && u == 1) {
        printf("These were the correct results.\n\n");
    } else {
        printf("Sorry. Your platform doesn't support Arithmetic\n");
        printf("Right-Shift, defined differently from Logical\n");
        printf("Right-Shift.\n\n");
        return -1;
    }

    //  Open file table.h in write mode
    FILE *fptr = fopen("table2.h", "w");

    if (fptr == NULL) {
        printf("Error: Unable to open file 'table.h' for writing!\n");
        return -1;
    }

    fprintf(fptr, "{\n");

    for (int i = 0; i < 256; i++) {
        temp = (i * M_PI) / 128.0;
        a = floor((sin(temp) * 16384.0) + 0.5);
        fprintf(fptr, "%d,\n", a);
    }

    fprintf(fptr, "};\n");

    fclose(fptr);

    a = M_PI * 65536.0 * 2.0;
    printf("Integer rep of Pi * 2, with 16 LSBs as fraction:\n");
    printf("%d\n\n", a);
    return 0;
}
